//=============================================================================
// DawnsEOGTrigger: senses things happening in its proximity and generates
// a call to LevelInfo.Game.EndGame(fraglimit) to end the game if the Touching
// Actor is the InfernoMap Conquerer.  Gives the Conquerer a sure-to-win score,
// but adjusts Time in Game to provide a reasonable FPH calculation.
// Sets Conqueror's Velocity to 0 to prevent him sliding at end of game
//
// Author:  Dawn Knight
// Tested only on versions 4.36 & 4.51
// Version A4.2  05/27/2004 09:35:50 AM
//=============================================================================
class DawnsEOGTrigger extends Trigger;

var InfernoMap IMap;

function PostBeginPlay()
{
	local InfernoMap I;

  // Find the InfernoMap Mutator if present
	foreach AllActors( class 'InfernoMap', I )
	{
		IMap = I;
		Log("**** DawnsEOGTrigger Found InfernoMap"@IMap);
	}
	Super.PostBeginPlay();
}

// Called when something touches the trigger.
//
function Touch( actor Other )
{
	local float CurrentSeconds, CurrentMinutes, FPH;

	super.Touch( Other );

	if (Level.Game.IsA('DeathMatchPlus') && Pawn(Other) != none)
	{
		if ( Pawn(Other) == IMap.Conquerer )
		{
			// Hack to set FPH calculation in GameInfo.EndGame().
			CurrentSeconds = Level.TimeSeconds - Pawn(Other).PlayerReplicationInfo.StartTime;
			CurrentMinutes = CurrentSeconds / 60;
			FPH = Pawn(Other).PlayerReplicationInfo.Score / (CurrentMinutes / 60);
			// Now, FPH is set to what we want for EndGame stats, but we are going to
			// set score much higher and need to alter .StartTime to compensate.
			if ( Pawn(Other).PlayerReplicationInfo.Score < 10 )
			{
				Pawn(Other).PlayerReplicationInfo.Score = 500;
			}
			else
			{
				Pawn(Other).PlayerReplicationInfo.Score = 1000;
			}
			Pawn(Other).PlayerReplicationInfo.StartTime = Level.TimeSeconds -
				(3600 * Pawn(Other).PlayerReplicationInfo.Score / FPH);
			Pawn(Other).PlayerReplicationInfo.Velocity = Vect(0.0,0.0,0.0);
			DeathMatchPlus(Level.Game).EndGame("fraglimit");
		}
	}
}

defaultproperties
{
     DrawScale=2
}
