//=============================================================================
// DawnsInfernoMatch.
// By Dawn Knight ver 1.02 $DateTime: Fri, 08/26/2005 09:41:08 am$
// A custom DeathMatchPlus for DM-DawnsInfernoF.
//=============================================================================
class DawnsInfernoMatch extends DawnsDeathMatchPlus
	config;

var InfernoMap IMap;
var PlayerStart ConquererStart;

function PreBeginPlay()
{
	local GameReplicationInfo GRI;
	local InfernoMap IM;
	local PlayerStart CS;

	StartTime = 0;
	SetGameSpeed(GameSpeed);
	Level.bNoCheating = bNoCheating;
	Level.bAllowFOV = bAllowFOV;

	if (GameReplicationInfoClass != None)
		GameReplicationInfo = Spawn(GameReplicationInfoClass);
	else
		GameReplicationInfo = Spawn(class'GameReplicationInfo');
	InitGameReplicationInfo();
	foreach AllActors( class'InfernoMap', IM )
	{
		IMap = IM;
		Log("****"@self@"found InfernoMap"@IMap);
		break;
	}
	foreach AllActors( class'PlayerStart', CS )
	{
		if( CS.TeamNumber == 61 )
		{
			ConquererStart = CS;
			Log("****"@self@"found ConquererStart"@ConquererStart);
			break;
		}
	}
}

function PostBeginPlay()
{
//	Log("**** DawnsInfernoMatch.PostBeginPlay");
	Super.PostBeginPlay();
//	Log("**** FragLimit"@FragLimit@"MaxPlayers"@MaxPlayers@"Difficulty"@Difficulty@"GameSpeed"@GameSpeed);
}

function PreCacheReferences()
{
	//never called - here to force precaching of meshes
	spawn(class'TMale1');
	spawn(class'TMale2');
	spawn(class'TFemale1');
	spawn(class'TFemale2');
	spawn(class'ImpactHammer');
	spawn(class'Translocator');
	spawn(class'Enforcer');
	spawn(class'UT_Biorifle');
	spawn(class'ShockRifle');
	spawn(class'PulseGun');
	spawn(class'Ripper');
	spawn(class'Minigun2');
	spawn(class'UT_FlakCannon');
	spawn(class'UT_Eightball');
	spawn(class'SniperRifle');
}

function CheckReady()
{
	super.CheckReady();
}

// Set gameplay speed.
//
function SetGameSpeed( Float T )
{
	GameSpeed = FMax(T, 0.1);
	if ( bHardCoreMode )
	{
		if ( bChallengeMode )
			Level.TimeDilation = 1.25 * GameSpeed;
		else
			Level.TimeDilation = 1.1 * GameSpeed;
	}
	else
		Level.TimeDilation = GameSpeed;
//	SaveConfig();
	SetTimer(Level.TimeDilation, true);
}

function PlayTeleportEffect( actor Incoming, bool bOut, bool bSound)
{
 	super.PlayTeleportEffect( Incoming, bOut, bSound);
}

// Parse options for this game...
// Never called if a Mutator substitutes this GameInfo in.
// Gets called if this is the default GameType
event InitGame( string Options, out string Error )
{
	Log("**** InitGame Start");
	Super.InitGame(Options, Error);
	Log("**** InitGame End");
}

// Set game settings based on ladder information when RatedPlayer logs in.
function InitRatedGame(LadderInventory LadderObj, PlayerPawn LadderPlayer)
{
	super.InitRatedGame( LadderObj, LadderPlayer);
}

function AcceptInventory(pawn PlayerPawn)
{
	super.AcceptInventory( PlayerPawn);
}

function bool SetEndCams(string Reason)
{
	local bool bResult;

	bResult = super.SetEndCams( Reason);
	return bResult;
}

function PlayWinMessage(PlayerPawn Player, bool bWinner)
{
	super.PlayWinMessage( Player, bWinner);
}

function NotifySpree(Pawn Other, int num)
{
	super.NotifySpree( Other, num);
}

function EndSpree(Pawn Killer, Pawn Other)
{
	super.EndSpree( Killer, Other);
}

function ScoreKill(pawn Killer, pawn Other)
{
	Other.DieCount++;
//	Log("**** DM.ScoreKill 1 Killer"@Killer@"Other"@Other);
	if( ((Killer == None) || (Killer == Other)) &&
			Other.PlayerReplicationInfo != none )
		Other.PlayerReplicationInfo.Score -= 1;
	else if ( Killer != None )
	{
		Killer.killCount++;
		if ( Killer.PlayerReplicationInfo != None )
			Killer.PlayerReplicationInfo.Score += 1;
	}
	BaseMutator.ScoreKill(Killer, Other);
}

// Monitor killed messages for fraglimit
function Killed(pawn killer, pawn Other, name damageType)
{
	Super.Killed(killer, Other, damageType);
}

event playerpawn Login
(
	string Portal,
	string Options,
	out string Error,
	class<playerpawn> SpawnClass
)
{
	local playerpawn MyNewPlayer;
	local GameReplicationInfo GRI;

//	Log("**** DamageMutator"@DamageMutator@"MessageMutator"@MessageMutator);
	MyNewPlayer = Super.Login(Portal, Options, Error, SpawnClass);
	return MyNewPlayer;
}

event PostLogin( playerpawn NewPlayer )
{
	Super.PostLogin(NewPlayer);
}

function int ReduceDamage(int Damage, name DamageType, pawn injured, pawn instigatedBy)
{
	local int MyDamage;

	MyDamage = super.ReduceDamage( Damage, DamageType, injured, instigatedBy);
	return MyDamage;
}

function StartMatch()
{
	Log("**** StartMatch");
	super.StartMatch();
}

function Timer()
{
	Super.Timer();
}

function bool TooManyBots()
{
	return (NumBots + NumPlayers > MinPlayers);
}

function bool RestartPlayer( pawn aPlayer )
{
	local bool bResult;

	bResult = Super.RestartPlayer(aPlayer);
	return bResult;
}

function SendStartMessage(PlayerPawn P)
{
	super.SendStartMessage( P);
}

function Bot SpawnBot(out NavigationPoint StartSpot)
{
	local bot MyBot;

	MyBot = super.SpawnBot( StartSpot);
	return MyBot;
}

function Bot SpawnRatedBot(out NavigationPoint StartSpot)
{
	local bot MyBot;

	MyBot = super.SpawnRatedBot( StartSpot);
	return MyBot;
}

function bool ForceAddBot()
{
	local bool bResult;

	bResult = super.ForceAddBot();
	return bResult;
}

function bool AddBot()
{
	local bool bResult;

	bResult = super.AddBot();
	return bResult;
}

function ModifyBehaviour(Bot NewBot);

function PlayStartUpMessage(PlayerPawn NewPlayer)
{
	super.PlayStartUpMessage( NewPlayer );
}

function float PlayerJumpZScaling()
{
	local float Result;

	Result = super.PlayerJumpZScaling();
	return Result;
}

function AddDefaultInventory( pawn PlayerPawn )
{
	Super.AddDefaultInventory(PlayerPawn);
}

function GiveWeapon(Pawn PlayerPawn, string aClassName )
{
	super.GiveWeapon( PlayerPawn, aClassName );
}

function byte AssessBotAttitude(Bot aBot, Pawn Other)
{
	local byte Result;

	Result = super.AssessBotAttitude( aBot, Other );
	return Result;
}

function float GameThreatAdd(Bot aBot, Pawn Other)
{
	return 0;
}

// AllowTranslocation - return true if Other can teleport to Dest
function bool AllowTranslocation(Pawn Other, vector Dest )
{
	return true;
}

function bool CanTranslocate(Bot aBot)
{
	if ( !bUseTranslocator || (bRatedGame && !bRatedTranslocator) )
		return false;
	return ( (aBot.MyTranslocator != None) && (aBot.MyTranslocator.TTarget == None) );
}

function PickAmbushSpotFor(Bot aBot)
{
	super.PickAmbushSpotFor( aBot );
}

function RateVs(Pawn Other, Pawn Killer)
{
	super.RateVs( Other, Killer );
}

function bool SuccessfulGame()
{
	local bool bResult;

	super.SuccessfulGame();
}

// Commented out for release version.
function Skip()
{
	super.Skip();
}

function SkipAll()
{
	super.SkipAll();
}

function bool CanSpectate( pawn Viewer, actor ViewTarget )
{
	local bool bResult;

	bResult = super.CanSpectate( Viewer, ViewTarget );
	return bResult;
}

function bool ShouldRespawn(Actor Other)
{
	local bool bResult;

	bResult = super.ShouldRespawn( Other );
	return bResult;
}

function ChangeName(Pawn Other, string S, bool bNameChange)
{
	super.ChangeName( Other, S, bNameChange );
}

// FindPlayerStart()
// Returns the 'best' player start for this player to start from.
function NavigationPoint FindPlayerStart(Pawn Player, optional byte InTeam, optional string incomingName)
{
	local PlayerStart Dest, Candidate[16], Best;
	local float Score[16], BestScore, NextDist;
	local pawn OtherPlayer;
	local int i, num;
	local Teleporter Tel;
	local NavigationPoint N, LastPlayerStartSpot;
	local byte StartType;

	if ( bStartMatch && (Player != None) && Player.IsA('TournamentPlayer')
			&& (Level.NetMode == NM_Standalone)
			&& (TournamentPlayer(Player).StartSpot != None) )
		return TournamentPlayer(Player).StartSpot;

	if( incomingName!="" )
		foreach AllActors( class 'Teleporter', Tel )
			if( string(Tel.Tag)~=incomingName )
				return Tel;

	// StartType defaults to 0 upon entry to function.
	if( Player != None && Player == IMap.Conquerer )
	{
		return ConquererStart; // 61: Conquerer always Spawns at this one location.
	}
	if( IMap.bSatanAlreadyKilled == true)
	{
		StartType = 62; // Everyone but Conquerer Spawns here after Satan Destroyed.
	}
	else
	{
		if( (Player != none) && Player.IsA('Bot') &&
				IMap.InnerSanctumGate.IsInState('Mover') )
		{
			StartType = 63; // Bots ReSpawn inside InnerSanctum once Gate is opened.
		}
	}
	//choose candidates
	foreach AllActors( class 'PlayerStart', Dest )
	{
		if ( (Dest.bEnabled && StartType == 0) ||
				(!Dest.bEnabled && (StartType != 0) && (StartType == Dest.TeamNumber)) )
		{
			if (num<16)
				Candidate[num] = Dest;
			else if (Rand(num) < 16)
				Candidate[Rand(16)] = Dest;
			num++;
		}
	}

	if (num>16) num = 16;
	else if (num == 0)
	{
		Log("**** FindPlayerStart: PlayerStart NOT Found!");
		return None;
	}

	if ( (Player != None) && Player.IsA('TournamentPlayer')
		&& (TournamentPlayer(Player).StartSpot != None) )
		LastPlayerStartSpot = TournamentPlayer(Player).StartSpot;

	//assess candidates
	for (i=0;i<num;i++)
	{
		if ( (Candidate[i] == LastStartSpot) || (Candidate[i] == LastPlayerStartSpot) )
			Score[i] = -10000.0;
		else
			Score[i] = 3000 * FRand(); //randomize
	}
	for ( OtherPlayer=Level.PawnList; OtherPlayer!=None; OtherPlayer=OtherPlayer.NextPawn)
		if ( OtherPlayer.bIsPlayer && (OtherPlayer.Health > 0) && !OtherPlayer.IsA('Spectator') )
			for ( i=0; i<num; i++ )
			{
				if ( OtherPlayer.Region.Zone == Candidate[i].Region.Zone )
				{
					Score[i] -= 1500;
					NextDist = VSize(OtherPlayer.Location - Candidate[i].Location);
					if ( NextDist < OtherPlayer.CollisionRadius + OtherPlayer.CollisionHeight )
						Score[i] -= 1000000.0;
					else if ( (NextDist < 2000) && FastTrace(Candidate[i].Location, OtherPlayer.Location) )
						Score[i] -= (10000.0 - NextDist);
				}
				else if ( NumPlayers + NumBots == 2 )
				{
					Score[i] += 2 * VSize(OtherPlayer.Location - Candidate[i].Location);
					if ( FastTrace(Candidate[i].Location, OtherPlayer.Location) )
						Score[i] -= 10000;
				}
			}

	BestScore = Score[0];
	Best = Candidate[0];
	for (i=1;i<num;i++)
		if (Score[i] > BestScore)
		{
			BestScore = Score[i];
			Best = Candidate[i];
		}

	LastStartSpot = Best;
	return Best;
}

function Logout(pawn Exiting)
{
	Super.Logout(Exiting);
}

function bool NeedPlayers()
{
	return (!bGameEnded && (NumPlayers + NumBots < MinPlayers));
}

function RestartGame()
{
	super.RestartGame();
}

function LogGameParameters(StatLog StatLog)
{
	Super.LogGameParameters(StatLog);
}

// Game Querying.
function string GetRules()
{
	local string ResultSet;

	ResultSet = Super.GetRules();
	return ResultSet;
}

function InitGameReplicationInfo()
{
	Super.InitGameReplicationInfo();
}

function bool CheckThisTranslocator(Bot aBot, TranslocatorTarget T)
{
	return false;
}

function bool OneOnOne()
{
	return ( NumPlayers + NumBots == 2 );
}

function float SpawnWait(bot B)
{
	local float Result;

	Result = super.SpawnWait( B );
	return Result;
}

function bool NeverStakeOut(bot Other)
{
	return false;
}

defaultproperties
{
     AirControl=0.350000
     FragLimit=0
     bHardCoreMode=True
     bMultiWeaponStay=True
     NetWait=10
     RestartWait=15
     bUseTranslocator=True
     CountDown=10
     TourneyMessage="Waiting for other players."
     WaitingMessage1="Waiting for ready signals."
     WaitingMessage2="(Use your fire button to toggle ready!)"
     ReadyMessage="You are READY!"
     NotReadyMessage="You are NOT READY!"
     CountDownMessage=" seconds until play starts!"
     StartMessage="The match has begun!"
     GameEndedMessage="wins the match!"
     SingleWaitingMessage="Press Fire to start."
     gamegoal="frags wins the match."
     InitialBots=4
     NoNameChange=" is already in use."
     OvertimeMessage="Score tied at the end of regulation. Sudden Death Overtime!!!"
     BotConfigType=Class'InfernoBotInfo'
     LadderTypeIndex=1
     bRestartLevel=False
     bPauseable=False
     bDeathMatch=True
     ScoreBoardType=Class'Botpack.TournamentScoreBoard'
     BotMenuType="UTMenu.UTBotConfigSClient"
     RulesMenuType="UTMenu.UTRulesSClient"
     SettingsMenuType="UTMenu.UTSettingsSClient"
     GameUMenuType="UTMenu.UTGameMenu"
     MultiplayerUMenuType="UTMenu.UTMultiplayerMenu"
     GameOptionsMenuType="UTMenu.UTOptionsMenu"
     HUDType=Class'Botpack.ChallengeHUD'
     MapListType=Class'Botpack.TDMmaplist'
     MapPrefix="DM"
     BeaconName="DM"
     GameName="Tournament DawnsInfernoMatch"
     DeathMessageClass=Class'Botpack.DeathMessagePlus'
     DMMessageClass=Class'Botpack.DeathMatchMessage'
     MutatorClass=Class'Botpack.DMMutator'
     GameReplicationInfoClass=Class'Botpack.TournamentGameReplicationInfo'
     bLoggingGame=True
}
