//=============================================================================
// InfernoMsgTrigger: senses things happening in its proximity and spawns
// a short-lived repeating message (InfernoTriggeredMsg) to Player(s).
// Author:  Dawn Knight
// Tested only on version 4.36
// Version A4.1  11/18/2003 10:33:19 PM
//=============================================================================
class InfernoMsgTrigger extends Trigger;

var() int MessageSwitch;
var() bool bMessageOnceOnly;
var bool bSendMessage;
var rotator RMsgSwitch;

function PostBeginPlay()
{
	// Have to pass MessageSwitch thru available arguments of Spawn func.
	RMsgSwitch.Yaw = MessageSwitch;
	bSendMessage = true;
	Super.PostBeginPlay();
}

// Called when something touches the trigger.
function Touch( actor Other )
{
	local PlayerPawn PP;
	local actor A;

	if( IsRelevant( Other ) )
	{
		if ( ReTriggerDelay > 0 )
		{
			if ( Level.TimeSeconds - TriggerTime < ReTriggerDelay )
				return;
			TriggerTime = Level.TimeSeconds;
		}
		// Broadcast the Trigger message to all matching actors.
		if( Event != '' )
			foreach AllActors( class 'Actor', A, Event )
				A.Trigger( Other, Other.Instigator );

		if ( Other.IsA('Pawn') && (Pawn(Other).SpecialGoal == self) )
			Pawn(Other).SpecialGoal = None;

		if( Message != "" )
			// Send a string message to the toucher.
			Other.Instigator.ClientMessage( Message );

		if ( Level.Game.IsA('DeathMatchPlus') && PlayerPawn(Other) != none
			&& bSendMessage )
		{
			PP = PlayerPawn(Other);

			// Pass the MessageSwitch thru SpawnRotation.Yaw - sneaky
			Spawn( class'InfernoTriggeredMsg', PP,,,RMsgSwitch ); // This Actor flashes
			                                                      // the message
			if ( bMessageOnceOnly )
				bSendMessage = false;
		}
		if( bTriggerOnceOnly )
			// Ignore future touches.
			SetCollision(False);
		else if ( RepeatTriggerTime > 0 )
			SetTimer(RepeatTriggerTime, false);

	}
}

