//=============================================================================
// InfernoMappack.InfernoGoblinbot.  By Dawn Knight ver 1.01 09/26/2004
// Preferably IMPLICITLY Embedded in a Map.
//
// Allows "Peace Zones" where Bots ignore each other
//=============================================================================
//=============================================================================
// To IMPLICITLY Embed this Bot in a map without INSTANTIATING it:
// (Instantiating it will cause the game engine to attempt to SPAWN it and run
// it around like a real Bot which is NOT the purpose of Embedding it)
//
// Load Map in UED.
//
// Load this Bot script AND another script which REFERENCES it using:
//   obj load file=InfernoMappack.u Package=MyLevel
// at the UED command line.  For instance, InfernoMappack.InfernoBotInfo
// REFERENCES this Bot script (class) so it will be loaded, too.
//
// EXPLICITLY place the Referencing script in the Map, i.e., InfernoBotInfo.
//
// Here's the trick:  Go to the System Folder now and change the extension
// of the InfernoMappack.u file to anything else, say, .uZZZ BEFORE compiling
// and saving the Map.
//
// Now compile and save the Map.
//
// UED will see the REFERENCE to this Bot script, and NOT find a corresponding
// Package for it in the System Folder, so it will keep a copy of the code in
// the Map instead, but without an explicit Instantiation of it!  The code is
// in the Map, but the game will NOT attempt to execute it and it will NOT
// attempt to SPAWN a Bot!
//=============================================================================

class InfernoGoblinBot extends TSkaarjBot;

var InfernoMap IMap;

function PreBeginPlay()
{
  super.PreBeginPlay();
  PrePivot.Z = 39.0 * DrawScale - default.CollisionHeight;
	foreach AllActors( class'InfernoMap', IMap )
	{
		break;
	}
}

function eAttitude AttitudeTo(Pawn Other)
{
	local byte result;

	if( DeathMatchPlus(Level.Game) != none )
	{
    if (self.Region.Zone.ZoneTag == 'PeaceZone')
    {
      return ATTITUDE_Friendly; // No fighting in a Peace Zone!
    }
    if( self.Region.Zone.ZoneTag == 'Catacombs' )
    {
    	if( (Other != none) && (Other == IMap.Conquerer) )
    		return ATTITUDE_Hate;
    	else
    		return ATTITUDE_Friendly;
    }
		result = DeathMatchPlus(Level.Game).AssessBotAttitude(self, Other);
		Switch (result)
		{
			case 0: return ATTITUDE_Fear;
			case 1: return ATTITUDE_Hate;
			case 2: return ATTITUDE_Ignore;
			case 3: return ATTITUDE_Friendly;
		}
	}

	if ( Level.Game.bTeamGame && (PlayerReplicationInfo.Team == Other.PlayerReplicationInfo.Team) )
		return ATTITUDE_Friendly; //teammate
	return ATTITUDE_Hate;
}

defaultproperties
{
     DefaultFace="Guardian"
     TeamSkin="T_skaarj_"
     DefaultCustomPackage="TSkMSkins."
     drown=Sound'UnrealI.Skaarj.SKPDrown1'
     breathagain=Sound'UnrealI.Skaarj.SKPGasp1'
     Footstep1=Sound'MultiMesh.Skaarj.SkWalk'
     Footstep2=Sound'MultiMesh.Skaarj.SkWalk'
     Footstep3=Sound'MultiMesh.Skaarj.SkWalk'
     HitSound3=Sound'UnrealI.Skaarj.SKPInjur3'
     HitSound4=Sound'UnrealI.Skaarj.SKPInjur4'
     Deaths(0)=Sound'UnrealI.Skaarj.SKPDeath1'
     Deaths(1)=Sound'UnrealI.Skaarj.SKPDeath2'
     Deaths(2)=Sound'UnrealI.Skaarj.SKPDeath3'
     Deaths(3)=None
     Deaths(4)=Sound'UnrealI.Skaarj.SKPDeath1'
     Deaths(5)=Sound'UnrealI.Skaarj.SKPDeath3'
     GaspSound=Sound'UnrealI.Skaarj.SKPGasp1'
     UWHit1=Sound'UnrealShare.Male.MUWHit1'
     UWHit2=Sound'UnrealShare.Male.MUWHit2'
     LandGrunt=Sound'UnrealI.Skaarj.Land1SK'
     JumpSound=Sound'UnrealI.Skaarj.SKPJump1'
     DefaultSkinName="TSkMSkins.Warr"
     SelectionMesh="EpicCustomModels.TSkM"
     HitSound1=Sound'UnrealI.Skaarj.SKPInjur1'
     HitSound2=Sound'UnrealI.Skaarj.SKPInjur2'
     MenuName="Skaarj Hybrid"
     VoiceType="MultiMesh.SkaarjVoice"
     LODBias=0.720000
     Mesh=LodMesh'EpicCustomModels.TSkM'
     DrawScale=0.641026
     Fatness=160
     AmbientGlow=8
     ScaleGlow=0.25
     Intelligence=BRAINS_HUMAN
     BaseEyeHeight=20.00000
     EyeHeight=20.00000
     CollisionRadius=10.000000
     CollisionHeight=25.000000
}
