//=============================================================================
// InfernoMappack.InfernoMap.
// *** for use as an Embedded Mutator in DM-DawnsInfernoF ONLY ***
// Removes the Player's Enforcer - weapon choice is configurable.
// Sets Timelimit to no timelimit.
// Sets FragLimit to no fraglimit.
// Sets TransLocator to not kill ScriptedPawns.
// Controls a trigger when the Satan WarLord is killed.
// Ends the game when Satan WarLord Killer passes through Pearly Gates.
// An Embedded Mutator for placing INSIDE a special UT Map.
// Author:  Dawn Knight
// Tested only on versions 4.36 and 4.51
// Version 1.02 $DateTime: Fri, 08/26/2005 12:31:56 pm$
//=============================================================================
// To embed this Mutator in a Map:
// 1. Make sure this Mutator is NOT listed in the "EditPackages" section of your
//    UnrealTournament.ini file!  This is VERY Important!
// 2. Make sure "InfernoMapPack.u" IS located in your UnrealTournament/System
//    Folder.
// 3. Start a fresh UED session and open your Map for editing.
// 4. In the Command entry field at the bottom of the UED window type:
//
//         obj load file=InfernoMapPack.u package=MyLevel
//
// 5. Open the Actor
//
//         Actor->Info->Mutator->InfernoMap
//
// 6. Highlight "InfernoMap" in the browser and then select a surface
//    somewhere in your map, right-click on the surface, and in the pop-up
//    menu that appears, choose "Add InfernoMap Here".  Place only ONE
//    instance of this Mutator in your Map.
// 7. The resulting Actor that is placed in your map can be double-clicked on
//    to bring up the "InfernoMap Properties" browser where you can select
//    the Enforcer Replacement weapon.
// 8. Build and Save your map - the embedded Mutator code with your settings
//    will be saved away in the map.
//
// If you subsequently delete the single instance of the Mutator from your map,
// and then reBuild and reSave, the Mutator code will be deleted from the map
// and you will have to repeat all these steps to embed it again.
//=============================================================================
class InfernoMap extends Mutator; //config (InfernoMap);

const VERSION = "1.02";
const MINSATANHITS = 8;

var bool bPreBPInitialized;
var string TheReplacement;  // Weapon that will replace Enforcer
var DeathMatchPlus DM;
enum ERWeaponType // Replacement Weapon type.
{
  No_Weapon,
  Chainsaw,
  Enforcer,
  DoubleEnforcer,
  ImpactHammer,
  Minigun2,
  PulseGun,
  Ripper,
  ShockRifle,
  SuperShockRifle,
  SniperRifle,
  UT_BioRifle,
  UT_Eightball,
  UT_FlakCannon,
  Redeemer
};

var int SatanHits;  // Number of times Satan gets hit by PlayerPawns
var Pawn Conquerer; // The Player that kills the Satan WarLord
var Mover InnerSanctumGate;  // The Gate to Inner Sanctum
var Trigger ThroneTrigger; // The Trigger that opens Gate behind Throne
var WarLord Satan;  // The Builtin WarLord in the map
var bool bSatanAlreadyKilled;  // Used in ScoreKill()
var Mover ExitDoor;  // The 1 of 4 randomly selected doors to Heaven.

// Variables which the Level designer can adjust:
var() int BotMaxFallDamage; // the max damage a Bot can incur in a fall
var() ERWeaponType EnforcerReplacement; // accessible to Level Designer
var byte WasWarped[256];  // Was Player sent to Catacombs once Satan was Destroyed?
var config int LastDoor;  // Last ExitDoor used (prevent same door twice in a row)

// PreBeginPlay will be used to Bootstrap load this Mutator by linking it
// into the MutatorList as the 1st Mutator in a possible chain of Mutators.
function PreBeginPlay()
{
	local int i;
	local bool LoggedAlready;
	local Mutator M, Previous;
	local Mover Mvr;
	local Trigger T;
	local ZoneInfo ZI;
	local WarLord WL;
	local DeathMatchPlus OldDM;
	local Mover DoorNo[4];  // The 4 doors that lead to stairway to Heaven.

	if ( bPreBPInitialized ) // older versions of UT call this function twice but we
		return;                // only want to run our code once!
	bPreBPInitialized = True;
	OldDM = DeathMatchPlus( Level.Game );
	if ( OldDM.IsA('DawnsDeathMatchPlus') )
	{
		Level.Game = Spawn( class'DawnsInfernoMatch' );
		if ( Level.Game == None )
		{
			Level.Game = Spawn( class'DawnsDMatchPlus' );
			DM = DawnsDMatchPlus(Level.Game);
		}
		else
		DM = DawnsInfernoMatch(Level.Game);
	}
	else
	{
		Level.Game = Spawn( class'DawnsDMatchPlus' );
		DM = DawnsDMatchPlus(Level.Game);
	}
	DM.BaseMutator = OldDM.BaseMutator;
	DMMutator( DM.BaseMutator ).MyGame = DM;
	OldDM.SetOwner( None );
	OldDM.Destroy();

  // Add the mutator by linking it into the MutatorList.
  // this Embedded Mutator makes sense to be 1st after the BaseMutator
  // in the Mutatorlist...
  // But first, check for and destroy an already loaded version
	for (M = Level.Game.BaseMutator; M != None; M = M.NextMutator)
	{
		if (GetItemName(string(M.class)) == GetItemName(string(Self.class))) //Already loaded version - Destroy it!
		{
      Previous.NextMutator = M.NextMutator;
      if (M != Self) // This is the other version
      {
        UnLinkDamageMutator(M); // do this BEFORE M.Destroy()!
        M.Destroy();
        LoggedAlready = True;
      }
    }
    else  // this will happen at least the 1st time thru the For Loop.
    {
      Previous = M; // So Previous can now be used as the Previous Mutator
                    // in the next pass thru the for Loop.
    }
  }
  if (!LoggedAlready)
  {
    Log("");
    Log("**** InfernoMap Version"@VERSION$": Embedded Mutator for DM-DawnsInferno");
    Log("");
  }

  // We need to Bootstrap this instance into the List
  Self.NextMutator = Level.Game.BaseMutator.NextMutator; // Make a place in the List
  Level.Game.BaseMutator.NextMutator = Self; // place it 1st after BaseMutator
  Level.Game.RegisterDamageMutator(self);
  TheReplacement = GetRWeaponType(); //What's the Enforcer replacement weapon?

  // Set game variables
	if (DM != none)
	{
		Level.Game.bNoMonsters = false;
		Level.Game.bHumansOnly = false;
		DM.FragLimit = 0;
		DM.TimeLimit = 0;
		DM.RemainingTime = 0;
	}

	// Find the InnerSanctum Gate
  foreach AllActors( class 'Mover', Mvr, 'GateLevel3' )
  {
  	InnerSanctumGate = Mvr;
  	break;
  }

  // Find the Throne Trigger
  foreach AllActors( class 'Trigger', T, 'WarLordDeath' )
  {
  	ThroneTrigger = T;
  	break;
  }

	// Find the 'AscendToHeaven' Zones and force their ZoneGravity.Z values
  foreach AllActors( class 'ZoneInfo', ZI, 'AscendToHeaven' )
  {
  	// Some Mutators may insist on altering ZoneGravity and we can't allow
  	// that in the special 'AscendToHeaven' Zones, so we override it here.
		ZI.ZoneGravity.Z = 200.0;
  }

  // Find the Builtin WarLord
  foreach AllActors( class 'WarLord', WL )
  {
  	if ( WL != none)
  	{
  		Satan = WL;
  		break;
  	}
  }
  if (Satan == none)
  	Log("**** Could NOT find the Satan WarLord in map! ****");

  // Find the 4 doors that lead to stairway to Heaven.
  foreach AllActors( class 'Mover', Mvr )
  {
  	if ( Left(string(Mvr.Tag), 6) ~= "DoorNo" )
  	{
  		DoorNo[i] = Mvr;
  		if ( i == 3 )
  			break;
  		i++;
  	}
  }
  // Randomly choose one of the Doors to open when Satan is destroyed.
  // float -> Int conversion uses Truncation!!!
  // Rand() function is not very random so don't use it.
	i = int(FRand() * Level.Millisecond) % 4;
	if ( i == LastDoor )
		i = (i+1)%4;
	LastDoor = i;
	SaveConfig();
  ExitDoor = DoorNo[i];
  Log("**** ExitDoor is"@ExitDoor.Tag);
}

function UnLinkDamageMutator(Mutator M)
{
  local int i, count;
  local Mutator C, Previous;

  for (C = Level.Game.DamageMutator; C != none; C = C.NextDamageMutator)
  {
    if (C == M)
    {
      count = i;
      break;
    }
    i++;
    Previous = C;
  }
  if (count == 0)
  {
    Level.Game.DamageMutator = Level.Game.DamageMutator.NextDamageMutator;
  }
  else
  {
    Previous.NextDamageMutator = C.NextDamageMutator;
  }
}

//==============================================================================
// We already added ourselves to the Mutator List in our PreBeginPlay, but
// GameInfo.InitGame() will erroneously add us again unless we intercept the
// call with our own AddMutator function and prevent it.
function AddMutator(Mutator M)
{
  if (M == Self)
  {
    return; // Don't add us.
  }
  super.AddMutator(M);  // keep the chain unbroken
}

function string GetRWeaponType()
{
  switch( EnforcerReplacement )  // accessible to Level Designer
  {
    case No_Weapon:
      return "Botpack.None";
    case Chainsaw:
      return "Botpack.Chainsaw";
    case Enforcer:
      return "Botpack.Enforcer";
    case DoubleEnforcer:
      return "Botpack.DoubleEnforcer";
    case ImpactHammer:
      return "Botpack.ImpactHammer";
    case Minigun2:
      return "Botpack.Minigun2";
    case PulseGun:
      return "Botpack.PulseGun";
    case Ripper:
      return "Botpack.Ripper";
    case ShockRifle:
      return "Botpack.ShockRifle";
    case SuperShockRifle:
      return "Botpack.SuperShockRifle";
    case SniperRifle:
      return "Botpack.SniperRifle";
    case UT_BioRifle:
      return "Botpack.UT_BioRifle";
    case UT_Eightball:
      return "Botpack.UT_Eightball";
    case UT_FlakCannon:
      return "Botpack.UT_FlakCannon";
    case Redeemer:
      return "Botpack.WarheadLauncher";
    default:
      return "Botpack.Chainsaw";
  }
}

// Called just before PlayerPawn is Spawned.  We use it to adjust a Player's
// Inventory to our liking
function ModifyPlayer( Pawn PlayerPawn )
{
  local Inventory TheWeapon, Inv;

  if (!PlayerPawn.IsA('Spectator'))
  {
    TheWeapon = PlayerPawn.FindInventoryType(class'Enforcer');
    if (TheWeapon != None)
    {
      TheWeapon.DropInventory(); // Remove it from Player's Inventroy
      TheWeapon.Destroy(); // Remove it from the game
    }

    TheWeapon = PlayerPawn.FindInventoryType(class'ImpactHammer');
    if (TheWeapon != None)
    {
      TheWeapon.DropInventory(); // Remove it from Player's Inventroy
      TheWeapon.Destroy(); // Remove it from the game
    }

    if (TheReplacement != "Botpack.None")
    {
      DM.GiveWeapon(PlayerPawn,TheReplacement);  // Give Replacement for Enforcer
    }
    // Once Inner Sanctum Gate is open, it goes to state 'Mover'.  Use this to
    if (PlayerPawn.IsA('Bot') && (InnerSanctumGate.IsInState('Mover')))
		{
			DM.GiveWeapon(PlayerPawn,"Botpack.ImpactHammer");
			DM.GiveWeapon(PlayerPawn,"Botpack.PulseGun");
			DM.GiveWeapon(PlayerPawn,"Botpack.ShockRifle");
			DM.GiveWeapon(PlayerPawn,"Botpack.UT_Eightball");

			for( Inv=PlayerPawn.Inventory; Inv!=None; Inv=Inv.Inventory )
			{
				if (Ammo(Inv)!=None)
				{
					Ammo(Inv).AmmoAmount  = 999;
					Ammo(Inv).MaxAmmo  = 999;
				}
			}
		}
	}

  if ( NextMutator != None )
    NextMutator.ModifyPlayer(PlayerPawn);
}

/******************************************************************************/
// To prevent Bots from too much Falling Damage
// To prevent Skaarj from taking damage from Bots
// To prevent Satan from taking damage from anything except PlayerPawns
function MutatorTakeDamage( out int ActualDamage, Pawn Victim, Pawn InstigatedBy,
	out Vector HitLocation, out Vector Momentum, name DamageType)
{
	if ( Victim != none )
	{
	  if ( Victim.IsA('Bot') && DamageType == 'Fell' && ActualDamage > BotMaxFallDamage
			&& !InnerSanctumGate.IsInState('Mover') ) // Gate hasn't opened yet
	  {
	    ActualDamage = BotMaxFallDamage;
	  }
	  else
	  if ( Victim.IsA('SkaarjLord') && (InstigatedBy != none) && InstigatedBy.IsA('Bot') )
	  {
	  	ActualDamage = 0;
	  }
	  else
	  if ( Victim == Satan )
		{
			if ( (InstigatedBy != none) && InstigatedBy.IsA('PlayerPawn') )
	  	{
	  		if (SatanHits < MINSATANHITS)
	  			ActualDamage = 0;
	  		else
	  		if ( DamageType == 'Jolted' || DamageType == 'RedeemerDeath')
	  			ActualDamage = ActualDamage / 8;
				SatanHits++;
	 		}
			else
				ActualDamage = 0;
		}
	}
// Deliberately do NOT pass MutatorTakeDamage to next Mutator(s)!
//	if ( NextDamageMutator != None )
//		NextDamageMutator.MutatorTakeDamage( ActualDamage, Victim, InstigatedBy, HitLocation, Momentum, DamageType );
}

/******************************************************************************/
// To prevent TeleFragging Scripted Pawns and fix WarLord QuickDeath Bug
function bool PreventDeath(Pawn Killed, Pawn Killer, name damageType, vector HitLocation)
{
  if (Killer != none && Killed.IsA('ScriptedPawn') && damageType == 'gibbed')
  {
    return true;
  }

  if (Killed == Satan && SatanHits < MINSATANHITS)
	{
		Killed.Health = Killed.default.Health;
		return true;
	}

// Deliberately do NOT pass PreventDeath to next Mutator(s)!
//	if ( NextMutator != None )
//		return NextMutator.PreventDeath(Killed,Killer, damageType,HitLocation);
	return false;
}

/******************************************************************************/
// To Activate Throne Trigger when Satan WarLord is killed and force gibs when
// he dies.  Without gibs, a dead WarLord looks very unrealistic.
function ScoreKill(Pawn Killer, Pawn Other)
{
	if ( Other != none && Other == Satan ) // Ignore non-Satan WarLords in Map!
	{
		if (bSatanAlreadyKilled)
			return;
		bSatanAlreadyKilled = true;
		Other.SpawnGibbedCarcass();
		Other.Destroy();
		Conquerer = Killer;
		ExitDoor.Trigger(self, Killer);
		ThroneTrigger.bInitiallyActive = true;
		DM.TimeLimit = 310;
		DM.RemainingTime = 310;
		SetTimer(1.0, true);
		BroadcastLocalizedMessage( class'InfernoMessage', 6, Conquerer.PlayerReplicationInfo );
	}
// Deliberately do NOT pass ScoreKill to next Mutator(s)!
//	if ( NextMutator != None )
//		NextMutator.ScoreKill(Killer, Other);
}

function Timer()
{
	local byte count;
	local pawn P;

	// Send 2 players every second to the Catacombs once Satan is Destroyed.
	for ( P=Level.PawnList; P!=None; P=P.nextPawn )
	{
		if ( P.bIsPlayer && !P.IsA('Spectator') && (P.Health > 0) &&
				(WasWarped[P.PlayerReplicationInfo.PlayerID] == 0) &&
				(P != Conquerer) && (P.Region.Zone.ZoneTag != 'Catacombs') )
		{
			WarpToCataCombs(P);
			WasWarped[P.PlayerReplicationInfo.PlayerID] = 1;
			count++;
		}
		if( count >= 2 )
			break;
	}
	switch (DM.RemainingTime)
	{
		case 305:
			BroadcastLocalizedMessage( class'InfernoMessage', 6, Conquerer.PlayerReplicationInfo );
			break;
		case 295:
		case 285:
		case 275:
		case 235:
		case 225:
		case 215:
		case 175:
		case 165:
		case 155:
		case 115:
		case 105:
		case 95:
		case 55:
		case 45:
		case 35:
		case 20:
		case 15:
			BroadcastLocalizedMessage( class'InfernoMessage', 7, Conquerer.PlayerReplicationInfo );
			break;
	}
}

function bool WarpToCataCombs( Pawn P )
{
	local NavigationPoint startSpot;
	local bool foundStart;

	if( DM.bRestartLevel && Level.NetMode!=NM_DedicatedServer && Level.NetMode!=NM_ListenServer )
		return true;

	startSpot = DM.FindPlayerStart(P);
	if( startSpot == None )
	{
		log(" Player start not found!!!");
		return false;
	}
	foundStart = P.SetLocation(startSpot.Location);
	if( foundStart )
	{
		startSpot.PlayTeleportEffect(P, true);
		P.SetRotation(startSpot.Rotation);
		P.ViewRotation = P.Rotation;
		P.Acceleration = vect(0,0,0);
		P.Velocity = vect(0,0,0);
		P.SetCollision( true, true, true );
		P.ClientSetLocation( startSpot.Location, startSpot.Rotation );
		P.bHidden = false;
		P.SoundDampening = P.Default.SoundDampening;
	}
	else
		log("****"@startspot@"Player start not useable!");
	return foundStart;
}

defaultproperties
{
     DrawScale=5.0
     EnforcerReplacement="Botpack.None"
     BotMaxFallDamage=50
}

