//=============================================================================
// InfernoMap_ShieldBelt.  By Dawn Knight ver 1.0 12/28/2002
// Preferably Embedded in a Map.
//
// Fixes Shieldbelt Z-axis offset on scaled InfernoBots with adjusted PrePivots.
// Simply adds PrePivot info to PickupFunction.
// PrePivot is a 3d offset of a mesh to a Pawn Location.
//=============================================================================
class InfernoMap_ShieldBelt extends UT_ShieldBelt;

function PickupFunction(Pawn Other)
{
	local Inventory I;

	MyEffect = Spawn(class'UT_ShieldBeltEffect', Other,,Other.Location, Other.Rotation);
	MyEffect.Mesh = Owner.Mesh;
	MyEffect.DrawScale = Owner.Drawscale;

	// Simply added this next line to the UT-Shieldbelt code
	MyEffect.PrePivot = Owner.PrePivot;
  // Now the Belt will line up on a scaled, prepivoted (offset) mesh

	if ( Level.Game.bTeamGame && (Other.PlayerReplicationInfo != None) )
		TeamNum = Other.PlayerReplicationInfo.Team;
	else
		TeamNum = 3;
	SetEffectTexture();

	I = Pawn(Owner).FindInventoryType(class'UT_Invisibility');
	if ( I != None )
		MyEffect.bHidden = true;

	// remove other armors
	for ( I=Owner.Inventory; I!=None; I=I.Inventory )
		if ( I.bIsAnArmor && (I != self) )
			I.Destroy();
}

defaultproperties
{
     TeamFireTextureStrings(0)="UnrealShare.Belt_fx.ShieldBelt.RedShield"
     TeamFireTextureStrings(1)="UnrealShare.Belt_fx.ShieldBelt.BlueShield"
     TeamFireTextureStrings(2)="UnrealShare.Belt_fx.ShieldBelt.Greenshield"
     TeamFireTextureStrings(3)="UnrealShare.Belt_fx.ShieldBelt.N_Shield"
     TeamTextureStrings(0)="UnrealShare.Belt_fx.ShieldBelt.newred"
     TeamTextureStrings(1)="UnrealShare.Belt_fx.ShieldBelt.newblue"
     TeamTextureStrings(2)="UnrealShare.Belt_fx.ShieldBelt.newgreen"
     TeamTextureStrings(3)="UnrealShare.Belt_fx.ShieldBelt.newgold"
     TeamFireTextures(0)=FireTexture'UnrealShare.Belt_fx.ShieldBelt.RedShield'
     TeamFireTextures(1)=FireTexture'UnrealShare.Belt_fx.ShieldBelt.BlueShield'
     TeamFireTextures(2)=FireTexture'UnrealShare.Belt_fx.ShieldBelt.Greenshield'
     TeamFireTextures(3)=FireTexture'UnrealShare.Belt_fx.ShieldBelt.N_Shield'
     TeamTextures(0)=Texture'UnrealShare.Belt_fx.ShieldBelt.newred'
     TeamTextures(1)=Texture'UnrealShare.Belt_fx.ShieldBelt.newblue'
     TeamTextures(2)=Texture'UnrealShare.Belt_fx.ShieldBelt.newgreen'
     TeamTextures(3)=Texture'UnrealShare.Belt_fx.ShieldBelt.newgold'
     bDisplayableInv=True
     PickupMessage="You got the Shield Belt."
     ItemName="ShieldBelt"
     RespawnTime=60.000000
     PickupViewMesh=LodMesh'Botpack.ShieldBeltMeshM'
     ProtectionType1=ProtectNone
     ProtectionType2=ProtectNone
     Charge=150
     ArmorAbsorption=100
     bIsAnArmor=True
     AbsorptionPriority=10
     MaxDesireability=3.000000
     PickupSound=Sound'UnrealShare.Pickups.BeltSnd'
     DeActivateSound=Sound'UnrealShare.Pickups.Sbelthe2'
     Icon=Texture'UnrealShare.Icons.I_ShieldBelt'
     bOwnerNoSee=True
     RemoteRole=ROLE_DumbProxy
     Mesh=LodMesh'Botpack.ShieldBeltMeshM'
     CollisionRadius=25.000000
     CollisionHeight=10.000000
}
